from matplotlib import pyplot as plt
import pandas as pd
from utils.csv_to_df import import_measurements
from utils.global_constants import FIGURES_SAVE_PATH


def plot_speed_information(tonni_params: bool = True):
    # Import csv file with speed information as dataframes
    if tonni_params:
        VELOCITIES_A0_DF = pd.read_csv("teflon_10mm_tonni_A_Lamb.txt")
        VELOCITIES_S0_DF = pd.read_csv("teflon_10mm_tonni_S_Lamb.txt")
        PLOT_TITLE = "Parameters from Tonni"
    else:
        VELOCITIES_A0_DF = pd.read_csv("simulated_speeds_for_A0.csv")
        VELOCITIES_S0_DF = pd.read_csv("simulated_speeds_for_S0.csv")
        PLOT_TITLE = "Parameters from the Dispersion Calculator"

    # Set font sizes a bit larger
    SMALL_SIZE = 15
    MEDIUM_SIZE = 18
    plt.rc("font", size=SMALL_SIZE)  # controls default text sizes
    plt.rc("axes", titlesize=MEDIUM_SIZE)  # fontsize of the axes title
    plt.rc("axes", labelsize=MEDIUM_SIZE)  # fontsize of the x and y labels
    plt.rc("xtick", labelsize=SMALL_SIZE)  # fontsize of the tick labels
    plt.rc("ytick", labelsize=SMALL_SIZE)  # fontsize of the tick labels
    plt.rc("legend", fontsize=SMALL_SIZE)  # legend fontsize
    plt.rc("figure", titlesize=MEDIUM_SIZE)  # fontsize of the figure title

    fig, ax = plt.subplots()
    VELOCITIES_A0_DF.plot(
        ax=ax,
        x="A0 f (kHz)",
        y=["A0 Phase velocity (m/ms)", "A0 Energy velocity (m/ms)"],
        xlim=(0, 100),
        style={
            "A0 Energy velocity (m/ms)": "--",
            "A0 Phase velocity (m/ms)": "C0",
        },
    )
    VELOCITIES_S0_DF.plot(
        ax=ax,
        x="S0 f (kHz)",
        y=["S0 Phase velocity (m/ms)", "S0 Energy velocity (m/ms)"],
        xlabel="Frequency (kHz)",
        ylabel="Velocity (m/ms)",
        grid=True,
        style={
            "S0 Energy velocity (m/ms)": "--",
            "S0 Phase velocity (m/ms)": "C1",
        },
    )
    ax.get_legend().remove()
    ax.legend(
        [
            "A0 Phase Velocity",
            "A0 Group Velocity",
            "S0 Phase Velocity",
            "S0 Group Velocity",
        ],
        loc="upper right",
    )
    plt.tight_layout()
    # Save figure as pdf file in the path specified by FIGURES_SAVE_PATH
    # fig.savefig(
    #     f"{FIGURES_SAVE_PATH}/phase_and_energy_speeds_for_A0_and_S0.pdf",
    #     bbox_inches="tight",
    # )

    fig, ax = plt.subplots()

    VELOCITIES_A0_DF.plot(
        ax=ax,
        x="A0 f (kHz)",
        y="A0 Wavelength (mm)",
        xlim=(0, 100),
        ylim=(0, 50),
    )

    VELOCITIES_S0_DF.plot(
        ax=ax,
        x="S0 f (kHz)",
        y="S0 Wavelength (mm)",
        xlabel="Frequency (kHz)",
        ylabel="Wavelength (mm)",
        legend=False,
        grid=True,
    )
    ax.legend(
        [
            "A0",
            "S0",
        ],
        loc="upper right",
    )
    plt.tight_layout()
    # fig.savefig(
    #     f"{FIGURES_SAVE_PATH}/wavelengths_for_A0_and_S0.pdf", bbox_inches="tight"
    # )

    return 0
